"use strict";
// Copyright IBM Corp. 2019. All Rights Reserved.
// Node module: @loopback/example-context
// This file is licensed under the MIT License.
// License text available at https://opensource.org/licenses/MIT
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const context_1 = require("@loopback/context");
/**
 * A strongly-typed binding key for current date
 */
const CURRENT_DATE = context_1.BindingKey.create('currentDate');
/**
 * A strongly-typed binding key for `Greeter`
 */
const GREETER = context_1.BindingKey.create('greeter');
/**
 * A factory function to return the current date
 */
const getCurrentDate = () => new Date();
/**
 * A class with dependency injection
 */
let Greeter = class Greeter {
    // Constructor based injection
    constructor(userName) {
        this.userName = userName;
    }
    hello() {
        return `[${this.created.toISOString()}] (${this.requestId}) Hello, ${this.userName}`;
    }
};
__decorate([
    context_1.inject(CURRENT_DATE),
    __metadata("design:type", Date)
], Greeter.prototype, "created", void 0);
__decorate([
    context_1.inject('requestId'),
    __metadata("design:type", String)
], Greeter.prototype, "requestId", void 0);
Greeter = __decorate([
    __param(0, context_1.inject('currentUser')),
    __metadata("design:paramtypes", [String])
], Greeter);
/**
 * A provider is similar as the factory function but it requires dependency
 * injection. As a result, it's declared as a class with `@inject.*` decorations
 * applied.
 */
let RequestIdProvider = class RequestIdProvider {
    // Injection of `url`
    constructor(url) {
        this.url = url;
    }
    // This method returns the resolved value for the binding
    value() {
        let id = RequestIdProvider.ids.get(this.url);
        if (id == null) {
            id = 1;
        }
        else {
            id++;
        }
        RequestIdProvider.ids.set(this.url, id);
        return `${this.url}#${id}`;
    }
};
RequestIdProvider.ids = new Map();
RequestIdProvider = __decorate([
    __param(0, context_1.inject('url')),
    __metadata("design:paramtypes", [String])
], RequestIdProvider);
async function main() {
    const ctx = new context_1.Context('request');
    // Set the current user to `John` (a constant value)
    ctx.bind('currentUser').to('John');
    // Set current url
    ctx.bind('url').to('/greet');
    // Set the current date to a factory function that creates the value
    ctx.bind(CURRENT_DATE).toDynamicValue(getCurrentDate);
    // Bind `hello` to a class from which the value is instantiated
    ctx.bind(GREETER).toClass(Greeter);
    // Set `requestId` to a provider class which has a `value()` method to
    // create the value. It's a specializd factory declared as a class to allow
    // dependency injection
    ctx.bind('requestId').toProvider(RequestIdProvider);
    // Bind `hello` as an alias to `GREETER`
    ctx.bind('hello').toAlias(GREETER);
    let greeter = await ctx.get(GREETER);
    console.log(greeter.hello());
    greeter = await ctx.get('hello');
    console.log(greeter.hello());
}
exports.main = main;
if (require.main === module) {
    main().catch(err => {
        console.error(err);
        process.exit(1);
    });
}
//# sourceMappingURL=binding-types.js.map