"use strict";
// Copyright IBM Corp. 2019. All Rights Reserved.
// Node module: @loopback/example-context
// This file is licensed under the MIT License.
// License text available at https://opensource.org/licenses/MIT
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const context_1 = require("@loopback/context");
let Greeter = class Greeter {
    constructor(prefix) {
        this.prefix = prefix;
    }
    greet(name) {
        return `[${this.prefix}] Hello, ${name}!`;
    }
};
Greeter = __decorate([
    __param(0, context_1.inject('prefix')),
    __metadata("design:paramtypes", [String])
], Greeter);
async function main() {
    const appCtx = new context_1.Context('app');
    // Create a context per request, with `appCtx` as the parent
    const requestCtx = new context_1.Context(appCtx, 'request');
    const greeterBinding = appCtx
        .bind('services.Greeter')
        .toClass(Greeter)
        .tag('greeter');
    // Set prefix to `app` at app context level
    appCtx.bind('prefix').to(appCtx.name);
    // Get a greeter from request context
    let greeter = await requestCtx.get(greeterBinding.key);
    // Inherit `prefix` from app context
    console.log(greeter.greet('John'));
    // Set `prefix` at request context level
    requestCtx.bind('prefix').to(requestCtx.name);
    greeter = await requestCtx.get(greeterBinding.key);
    // Now the request context prefix is used
    console.log(greeter.greet('John'));
    // Get a greeter from app context
    greeter = await appCtx.get(greeterBinding.key);
    // Now the app context prefix is used
    console.log(greeter.greet('John'));
}
exports.main = main;
if (require.main === module) {
    main().catch(err => {
        console.error(err);
        process.exit(1);
    });
}
//# sourceMappingURL=context-chain.js.map