"use strict";
// Copyright IBM Corp. 2019. All Rights Reserved.
// Node module: @loopback/example-context
// This file is licensed under the MIT License.
// License text available at https://opensource.org/licenses/MIT
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const context_1 = require("@loopback/context");
const asGreeter = binding => {
    binding.tag('greeter');
};
const greeterFilter = binding => binding.tagMap['greeter'] != null;
class ChineseGreeter {
    constructor() {
        this.language = 'zh';
    }
    greet(name) {
        return `你好，${name}！`;
    }
}
let EnglishGreeter = class EnglishGreeter {
    constructor() {
        this.language = 'en';
    }
    greet(name) {
        return `Hello, ${name}!`;
    }
};
EnglishGreeter = __decorate([
    context_1.bind(asGreeter)
], EnglishGreeter);
async function main() {
    const ctx = new context_1.Context('request');
    // Add EnglishGreeter for now
    ctx.add(context_1.createBindingFromClass(EnglishGreeter, { namespace: 'greeters' }));
    // Add ChineseGreeter
    ctx
        .bind('greeters.ChineseGreeter')
        .toClass(ChineseGreeter)
        .tag('greeter');
    const enlishGreeterBinding = ctx.getBinding('greeters.EnglishGreeter');
    console.log(enlishGreeterBinding.key);
    let possibleEnglishGreeters = ctx.find('*.EnglishGreeter');
    console.log(possibleEnglishGreeters.map(b => b.key));
    possibleEnglishGreeters = ctx.find(/\w+\.EnglishGreeter$/);
    console.log(possibleEnglishGreeters.map(b => b.key));
    let greeterBindings = ctx.findByTag('greeter');
    console.log(greeterBindings.map(b => b.key));
    greeterBindings = ctx.find(context_1.filterByTag('greeter'));
    console.log(greeterBindings.map(b => b.key));
    greeterBindings = ctx.find(greeterFilter);
    console.log(greeterBindings.map(b => b.key));
    const view = ctx.createView(greeterFilter, (b1, b2) => b1.key.localeCompare(b2.key));
    console.log(view.bindings.map(b => b.key));
}
exports.main = main;
if (require.main === module) {
    main().catch(err => {
        console.error(err);
        process.exit(1);
    });
}
//# sourceMappingURL=find-bindings.js.map