"use strict";
// Copyright IBM Corp. 2019. All Rights Reserved.
// Node module: @loopback/example-context
// This file is licensed under the MIT License.
// License text available at https://opensource.org/licenses/MIT
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const context_1 = require("@loopback/context");
const REQUEST_CONTEXT = context_1.BindingKey.create('request.context');
const REQUEST_ID = context_1.BindingKey.create('tracing.requestId');
const REQUEST_ID_GENERATOR = context_1.BindingKey.create('tracing.requestIdGenerator');
const TRACING_INTERCEPTOR = context_1.BindingKey.create('tracing.interceptor');
const CONVERTER = context_1.BindingKey.create('converter');
const GREETER = context_1.BindingKey.create('greeter');
let TracingInterceptor = class TracingInterceptor {
    constructor(generator) {
        this.generator = generator;
    }
    value() {
        return this.intercept.bind(this);
    }
    async intercept(invocationCtx, next) {
        const reqCtx = invocationCtx.getSync(REQUEST_CONTEXT);
        let reqId = await reqCtx.get(REQUEST_ID, { optional: true });
        if (!reqId) {
            reqId = this.generator(reqCtx);
            // We need to set request id to `reqCtx` as `invocationCtx` is transient
            reqCtx.bind(REQUEST_ID).to(reqId);
            console.log('Adding request id at %s: %s', invocationCtx.targetName, reqId);
        }
        else {
            console.log('Request id found at %s: %s', invocationCtx.targetName, reqId);
        }
        return next();
    }
};
TracingInterceptor = __decorate([
    context_1.bind(context_1.asGlobalInterceptor('tracing')),
    __param(0, context_1.inject(REQUEST_ID_GENERATOR)),
    __metadata("design:paramtypes", [Function])
], TracingInterceptor);
class Converter {
    toUpperCase(name) {
        return name.toUpperCase();
    }
}
class Greeter {
    async greet(name) {
        const msg = await this.converter.toUpperCase(name);
        return `Hello, ${msg}`;
    }
}
__decorate([
    context_1.inject(CONVERTER, { asProxyWithInterceptors: true }),
    __metadata("design:type", Object)
], Greeter.prototype, "converter", void 0);
async function main() {
    const ctx = new context_1.Context('request');
    // Bind request context
    ctx.bind(REQUEST_CONTEXT).to(ctx);
    const binding = context_1.createBindingFromClass(TracingInterceptor, {
        key: TRACING_INTERCEPTOR,
    });
    ctx.add(binding);
    let count = 1;
    const reqUuidGenerator = context => `[${context.name}] ${count++}`;
    ctx.bind(REQUEST_ID_GENERATOR).to(reqUuidGenerator);
    ctx.bind(GREETER).toClass(Greeter);
    ctx
        .bind(CONVERTER)
        .toClass(Converter)
        .tag(context_1.BindingScope.SINGLETON);
    const greeter = await ctx.get(GREETER, { asProxyWithInterceptors: true });
    console.log(await greeter.greet('John'));
}
exports.main = main;
if (require.main === module) {
    main().catch(err => {
        console.error(err);
        process.exit(1);
    });
}
//# sourceMappingURL=interceptor-proxy.js.map