"use strict";
// Copyright IBM Corp. 2019. All Rights Reserved.
// Node module: @loopback/example-context
// This file is licensed under the MIT License.
// License text available at https://opensource.org/licenses/MIT
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const context_1 = require("@loopback/context");
/**
 * A function to create a new class with parameterized decorations
 * @param bindingKeyForName Binding key for current user
 * @param tags Additional binding tags
 */
function createClassWithDecoration(bindingKeyForName, ...tags) {
    let GreeterTemplate = class GreeterTemplate {
        constructor(userName) {
            this.userName = userName;
        }
        hello() {
            return `Hello, ${this.userName}`;
        }
    };
    GreeterTemplate = __decorate([
        context_1.bind({ tags }),
        __param(0, context_1.inject(bindingKeyForName)),
        __metadata("design:paramtypes", [String])
    ], GreeterTemplate);
    return GreeterTemplate;
}
async function main() {
    const ctx = new context_1.Context();
    ctx.bind('name1').to('John');
    ctx.bind('name2').to('Jane');
    const class1 = createClassWithDecoration('name1', { tags: { prefix: '1' } });
    const binding1 = context_1.createBindingFromClass(class1, { key: 'greeter1' });
    ctx.add(binding1);
    console.log('1:', binding1.tagMap);
    const class2 = createClassWithDecoration('name2', { tags: { prefix: '2' } });
    const binding2 = context_1.createBindingFromClass(class2, { key: 'greeter2' });
    ctx.add(binding2);
    console.log('2:', binding2.tagMap);
    const greeting1 = await ctx.get('greeter1');
    console.log('1: %s', greeting1.hello());
    const greeting2 = await ctx.get('greeter2');
    console.log('2: %s', greeting2.hello());
}
exports.main = main;
if (require.main === module) {
    main().catch(err => {
        console.error(err);
        process.exit(1);
    });
}
//# sourceMappingURL=parameterized-decoration.js.map